@@ -1,5 +1,5 @@
-from typing import List
-from overrides import overrides
+from typing import List, Dict, Any
+
 
 import spacy
 
@@ -44,25 +44,40 @@ class SpacySentenceSplitter(SentenceSplitter):
     """
 
     def __init__(self, language: str = "en_core_web_sm", rule_based: bool = False) -> None:
+        self._language = language
+        self._rule_based = rule_based
+
         # we need spacy's dependency parser if we're not using rule-based sentence boundary detection.
-        self.spacy = get_spacy_model(language, parse=not rule_based, ner=False, pos_tags=False)
+        self.spacy = get_spacy_model(self._language, parse=not self._rule_based, ner=False)
+        self._is_version_3 = spacy.__version__ >= "3.0"
         if rule_based:
             # we use `sentencizer`, a built-in spacy module for rule-based sentence boundary detection.
             # depending on the spacy version, it could be called 'sentencizer' or 'sbd'
             sbd_name = "sbd" if spacy.__version__ < "2.1" else "sentencizer"
             if not self.spacy.has_pipe(sbd_name):
-                sbd = self.spacy.create_pipe(sbd_name)
-                self.spacy.add_pipe(sbd)
+                if self._is_version_3:
+                    self.spacy.add_pipe(sbd_name)
+                else:
+                    sbd = self.spacy.create_pipe(sbd_name)
+                    self.spacy.add_pipe(sbd)
 
-    @overrides
     def split_sentences(self, text: str) -> List[str]:
-        return [sent.string.strip() for sent in self.spacy(text).sents]
+        if self._is_version_3:
+            return [sent.text.strip() for sent in self.spacy(text).sents]
+        else:
+            return [sent.string.strip() for sent in self.spacy(text).sents]
 
-    @overrides
     def batch_split_sentences(self, texts: List[str]) -> List[List[str]]:
         """
         This method lets you take advantage of spacy's batch processing.
         """
+        if self._is_version_3:
+            return [
+                [sentence.text.strip() for sentence in doc.sents] for doc in self.spacy.pipe(texts)
+            ]
         return [
             [sentence.string.strip() for sentence in doc.sents] for doc in self.spacy.pipe(texts)
         ]
+
+    def _to_params(self) -> Dict[str, Any]:
+        return {"type": "spacy", "language": self._language, "rule_based": self._rule_based}
