@@ -17,7 +17,11 @@ from allennlp.training.metrics.evalb_bracketing_scorer import (
     DEFAULT_EVALB_DIR,
 )
 from allennlp.training.metrics.fbeta_measure import FBetaMeasure
-from allennlp.training.metrics.fbeta_multi_label_measure import FBetaMultiLabelMeasure
+from allennlp.training.metrics.fbeta_verbose_measure import FBetaVerboseMeasure
+from allennlp.training.metrics.fbeta_multi_label_measure import (
+    FBetaMultiLabelMeasure,
+    F1MultiLabelMeasure,
+)
 from allennlp.training.metrics.f1_measure import F1Measure
 from allennlp.training.metrics.mean_absolute_error import MeanAbsoluteError
 from allennlp.training.metrics.metric import Metric
